<?php

namespace Argus\Discord;

use Argus\Uninstall;
use stdClass;


class Activate extends Uninstall {


    static function activate () {
        global $wpdb;

        if (!get_option('argusDiscordOAuth2AccessToken')) {

            self::removePluginDBTables(self::$discordDBTables);


            $optionTableVariables = [
                ['argusDiscordOAuth2State', ''],
                ['argusDiscordOAuth2TokenResponse', ''],
                ['argusDiscordOAuth2AccessToken', ''],
                ['argusDiscordWebhook', ''],
                ['argusDiscordGuild', ''],
                ['argusDiscordErrorMessage', ''],
                ['argusDiscordDataProfile', serialize(new stdClass())],
                ['argusDiscordDataApp', serialize(new stdClass())],
                ['argusDiscordDataGuild', serialize(new stdClass())],
                ['argusDiscordStylingOAuthButtons', 1],
                ['argusDiscordBlockNotVerified', 1],
                ['argusDiscordGuildFilterMode', 'disable'],
                ['argusDiscordRolesFilterMode', 'disable'],
                ['argusDiscordCustomRedirects', '{}'],
                ['argusDiscordUltimateMemberAccountPageIcon', 'um-faicon-link'],
            ];

            foreach ($optionTableVariables as $var) add_option($var[0], $var[1]);


            $tableQueries = [
                'CREATE TABLE `' . $wpdb->prefix . "argus_discord_messages` (
                    `id` BIGINT UNSIGNED NOT NULL,
                    `channel_id` BIGINT UNSIGNED NOT NULL,
                    `pinned` BOOLEAN NOT NULL DEFAULT FALSE,
                    `timestamp` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP,
                    `object` TEXT NOT NULL COMMENT 'Message object from Discord API',
                    PRIMARY KEY (`id`),
                    INDEX `CHANNEL_ID` (`channel_id`),
                    INDEX `PINNED` (`pinned`),
                    INDEX `TIMESTAMP` (`timestamp`)
                ) ENGINE = InnoDB DEFAULT CHARSET = utf8mb4 COLLATE utf8mb4_unicode_ci;",

                'CREATE TABLE `' . $wpdb->prefix . "argus_discord_assigned_roles` (
                    `user_id` BIGINT UNSIGNED NOT NULL,
                    `role_id` BIGINT UNSIGNED NOT NULL,
                    PRIMARY KEY (`user_id`, `role_id`)
                ) ENGINE = InnoDB DEFAULT CHARSET = utf8mb4 COLLATE utf8mb4_unicode_ci;",

                'CREATE TABLE `' . $wpdb->prefix . "argus_discord_members` (
                    `user_id` BIGINT UNSIGNED NOT NULL,
                    `object` TEXT NOT NULL COMMENT 'User object from Discord API',
                    PRIMARY KEY (`user_id`)
                ) ENGINE = InnoDB DEFAULT CHARSET = utf8mb4 COLLATE utf8mb4_unicode_ci;",

                'CREATE TABLE `' . $wpdb->prefix . "argus_discord_collected_data` (
                    `user_id` BIGINT UNSIGNED NOT NULL,
                    `object` TEXT NULL COMMENT 'User object from Discord API',
                    PRIMARY KEY (`user_id`)
                ) ENGINE = InnoDB DEFAULT CHARSET = utf8mb4 COLLATE utf8mb4_unicode_ci;",

                'CREATE TABLE `' . $wpdb->prefix . "argus_discord_channels` (
                    `channel_id` BIGINT UNSIGNED NOT NULL,
                    `object` TEXT NOT NULL COMMENT 'Channel object from Discord API',
                    `invite_object` TEXT NOT NULL COMMENT 'Invite object from Discord API',
                    PRIMARY KEY (`channel_id`)
                ) ENGINE = InnoDB DEFAULT CHARSET = utf8mb4 COLLATE utf8mb4_unicode_ci;",

                'CREATE TABLE `' . $wpdb->prefix . "argus_discord_roles` (
                    `role_id` BIGINT UNSIGNED NOT NULL,
                    `object` TEXT NOT NULL COMMENT 'Role object from Discord API',
                    PRIMARY KEY (`role_id`)
                ) ENGINE = InnoDB DEFAULT CHARSET = utf8mb4 COLLATE utf8mb4_unicode_ci;",

                'CREATE TABLE `' . $wpdb->prefix . "argus_discord_bound_roles` (
                    `wp_role` VARCHAR(110) NOT NULL,
                    `direction` TINYINT NOT NULL DEFAULT '1',
                    `discord_role` BIGINT UNSIGNED NOT NULL,
                    PRIMARY KEY (`wp_role`, `direction`, `discord_role`)
                ) ENGINE = InnoDB DEFAULT CHARSET = utf8mb4 COLLATE utf8mb4_unicode_ci;",
            ];

            foreach ($tableQueries as $query) $wpdb->query($query);


        } else {
            Cron::schedule();
        }

    }


}
