<?php

namespace Argus\Discord;


class Authorization extends Data {


    static function authorizePlugin () {

        // Authorization attempt
        if (!get_option('argusDiscordOAuth2State')) {
            update_option('argusDiscordOAuth2State', uniqid());

            add_action('wp_loaded', function() {
                wp_redirect('https://discordapp.com/api/oauth2/authorize?' . http_build_query([
                        'client_id'     => get_option('argusDiscordOAuth2ClientID'),
                        'state'         => get_option('argusDiscordOAuth2State'),
                        'response_type' => 'code',
                        'scope'         => 'identify bot guilds webhook.incoming',
                        'permissions'   => 805362691, // Current permissions: Manage Roles, Kick Members, Create Instant Invite, Manage Webhooks, Read Messages/View Channels, Send Messages, Send TTS Messages, Embed Links, Attach Files
                    ]));
            });

            // Authorization attempt failed
        } elseif (!wp_doing_ajax()) {
            update_option('argusDiscordOAuth2State', '');
            update_option('argusDiscordErrorMessage', __("During the last authorization attempt, wrong Client ID was passed.", 'argus-wordpress-plugin'));
            self::authorizationLogout(false);
        }
    }


    static function checkAuthorizationTokens () {
        self::fetchBearerTokenAndCheckBotToken($_GET['code'], get_option('argusDiscordOAuth2ClientID'), get_option('argusDiscordOAuth2ClientSecret'));
    }


    static function shareContent () {
        $redirectURL = $_COOKIE['argusLastPage'] ?? '/';

        // User canceled authorization or other error accrued
        if (isset($_GET['error'])) {
            $message = __("Guild or channel was not selected", 'argus-wordpress-plugin');
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Getting token
        $pluginData    = new Data();
        $tokenResponse = $pluginData->getBearerToken($_GET['code']);

        // If token was not received
        if (!isset($tokenResponse->webhook->url)) {
            $message = $_GET['error_description'] ?? __("Access token was not granted by the guild", 'argus-wordpress-plugin');
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Sending message
        $sharedPageID = str_replace('argus-discord-share-page-', '', $_GET['state']);
        $post         = get_post($sharedPageID);

        $message = '';
        if (isset($post->post_title)) $message .= '**' . $post->post_title . '**';
        if (isset($post->ID)) $message .= "\n" . get_permalink($post->ID);

        if ($message) Data::sendMessage(wp_strip_all_tags($message), $tokenResponse->webhook->url);

        wp_redirect($redirectURL);
        exit();
    }


    static function authorizeOrRegisterUser ($redirectTo = 'argusLastPage') {
        if ($redirectTo == 'argusPrevPage') {
            $redirectURL = $_COOKIE['argusPrevPage'] ?? '/';
        } else {
            $redirectURL = $_COOKIE['argusLastPage'] ?? '/';
        }

        // User canceled authorization or other error accrued
        if (isset($_GET['error'])) {
            $message = $_GET['error_description'] ?? '';
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Getting token and basic user data
        $pluginData       = new Data();
        $tokenResponse    = $pluginData->getBearerToken($_GET['code']);
        $responseUserData = $pluginData->getAuthorizedProfileData(['argusDiscordOAuth2AccessToken' => $tokenResponse->access_token]);

        // Checking if required data is received
        if (!isset($responseUserData->id) || !isset($responseUserData->username) || !isset($responseUserData->discriminator) || !isset($responseUserData->email) || !isset($responseUserData->verified)) {
            $message = __("Discord server did not send a requested data", 'argus-wordpress-plugin');
            if (isset($responseUserData->message)) {
                $message .= '. ' . __("Response message:", 'argus-wordpress-plugin');
                $message .= ' <i>' . $responseUserData->message . '</i>';
            }
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Checking authorization filters
        self::checkAuthorizationFilters($responseUserData, $tokenResponse, $redirectURL);

        // Registration or authorization

        // Looking for attached Discord profiles by ID
        $usersWithIDInMeta = get_users([
            'meta_key'   => 'dwpp-discord-id',
            'meta_value' => $responseUserData->id
        ]);

        // Registering new user
        if (!email_exists($responseUserData->email) && !count($usersWithIDInMeta)) {

            // Error if user registration are off
            if (!get_option('users_can_register')) {
                $message = __("Registration of new users is disabled", 'argus-wordpress-plugin');
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }

            // Selecting name
            if (!username_exists($responseUserData->username)) {
                $name = $responseUserData->username;
            } elseif (!username_exists($responseUserData->username . $responseUserData->discriminator)) {
                $name = $responseUserData->username . $responseUserData->discriminator;
            } else {
                $name = $responseUserData->username . time();
            }

            $registerResult = register_new_user(sanitize_user($name, true), $responseUserData->email);
            if (isset($registerResult->errors)) {
                $message = \Argus\Data::unwrapArrayInString($registerResult->errors);
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
        }

        // Selecting user for authorization
        $user = get_user_by('email', $responseUserData->email);
        if (!$user) $user = $usersWithIDInMeta[0];

        // Processing authorization
        update_user_meta($user->ID, 'dwpp-discord-id', $responseUserData->id);
        $pluginData->saveCollectedUserData($responseUserData->id, $responseUserData);
        $pluginData->fetchSingleMemberData($responseUserData->id);
        RolesSync::syncUserRoles($user->ID);
        wp_set_auth_cookie($user->ID, true);

        // Defining final redirect page location
        if ($redirectTo == 'argusLastPage' || $redirectTo == 'argusPrevPage') {
            $finalRedirectURL = $redirectURL;
        } else {
            $redirectArray = json_decode(get_option('argusDiscordCustomRedirects'));
            preg_match('/^argus-auth-discord-go-([0-9]+)-page$/', $redirectTo, $parsedRedirectState);
            if (isset($redirectArray->{"$parsedRedirectState[1]"}) && isset($redirectArray->{"$parsedRedirectState[1]"}->url) && $redirectArray->{"$parsedRedirectState[1]"}->url) {
                $finalRedirectURL = $redirectArray->{"$parsedRedirectState[1]"}->url;
            } else {
                $finalRedirectURL = '/';
            }
        }

        // Checking if redirect after authorization is active
        if (get_option('argusDiscordNewUserRedirect')) {

            // Checking if redirect only for users that are not in the guild and this user is in the guild
            if (get_option('argusDiscordNewUserRedirectIfNotInGuild') && $pluginData->checkIfUserInGuild($responseUserData->id)) {
                $redirectURL = $finalRedirectURL;

                // If redirect only for users that are not in the guild disabled or user are not in the guild
            } else {
                $redirectURL = get_option('argusDiscordNewUserRedirect');
            }

            // If redirect after authorization is disabled
        } else {
            $redirectURL = $finalRedirectURL;
        }

        // If option to open redirect in new tab is active
        if (get_option('argusDiscordNewUserRedirectInNewTab')) {
            ?>
            <script>
                newWindow = window.open('<?php echo $redirectURL; ?>', '_blank');
                if ((newWindow == null) || (newWindow.closed)) {
                    window.location = '<?php echo $redirectURL; ?>';
                } else {
                    window.location = '<?php echo $_COOKIE['argusLastPage'] ?? '/'; ?>';
                }
            </script>
            <?php

            // If option to open redirect in new tab is disabled
        } else {
            wp_redirect($redirectURL);
        }
        exit();
    }


    static function connectProfileToUser () {
        $redirectURL = $_COOKIE['argusLastPage'] ?? '/';

        // User canceled authorization or other error accrued
        if (isset($_GET['error'])) {
            $message = $_GET['error_description'] ?? '';
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Getting token and basic user data
        $pluginData       = new Data();
        $tokenResponse    = $pluginData->getBearerToken($_GET['code']);
        $responseUserData = $pluginData->getAuthorizedProfileData(['argusDiscordOAuth2AccessToken' => $tokenResponse->access_token]);
        $wpUserID         = get_current_user_id();

        // Checking if required data is received
        if (!isset($responseUserData->id) || !isset($responseUserData->username) || !isset($responseUserData->discriminator) || !isset($responseUserData->email) || !isset($responseUserData->verified)) {
            $message = __("Discord server did not send a requested data", 'argus-wordpress-plugin');
            if (isset($responseUserData->message)) {
                $message .= '. ' . __("Response message", 'argus-wordpress-plugin');
                $message .= ' <i>' . $responseUserData->message . '</i>';
            }
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Checking authorization filters
        if (get_option('argusDiscordFilterProfileConnection')) {
            self::checkAuthorizationFilters($responseUserData, $tokenResponse, $redirectURL);
        }

        // Processing profile connection
        if ($wpUserID) {
            update_user_meta($wpUserID, 'dwpp-discord-id', $responseUserData->id);
            $pluginData->saveCollectedUserData($responseUserData->id, $responseUserData);
            $pluginData->fetchSingleMemberData($responseUserData->id, true);
            if (get_option('argusDiscordAssignRoleIfProfileConnected')) {
                (new WP_User($wpUserID))->add_role(get_option('argusDiscordAssignRoleIfProfileConnected'));
            }
            RolesSync::syncUserRoles($wpUserID);
        }

        if (get_option('argusDiscordProfileConnectedRedirect')) $redirectURL = get_option('argusDiscordProfileConnectedRedirect');
        wp_redirect($redirectURL);
        exit();
    }


    static function disconnectDiscordProfileFromUser () {

        $wpUserID = get_current_user_id();

        if ($wpUserID) {
            if (get_option('argusDiscordDeleteFromGuildIfProfileDisconnected')) {
                Data::removeUserFromServer(get_user_meta($wpUserID, 'dwpp-discord-id', true), $wpUserID);
            }
            update_user_meta($wpUserID, 'dwpp-discord-id', '');
        }

        wp_redirect($_COOKIE['argusLastPage'] ?? '/');
        exit();
    }


    private static function checkAuthorizationFilters ($responseUserData, $tokenResponse, $redirectURL) {
        $pluginData = new Data();

        // Filtering not verified user
        if (get_option('argusDiscordBlockNotVerified') && !$responseUserData->verified) {
            $message = __("Your Discord account is not verified", 'argus-wordpress-plugin');
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Filtering empty email
        if (!isset($responseUserData->email) || !$responseUserData->email) {
            $message = __("Discord account must have a connected email", 'argus-wordpress-plugin');
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Looking for a member in your guild
        $memberUserData = $pluginData->getSingleMemberData($responseUserData->id, true);

        // Filtering pending users
        if (get_option('argusDiscordBlockPending')) {
            if (!isset($memberUserData->pending)) {
                $message = __("Your are not a member of the guild", 'argus-wordpress-plugin');
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
            if ($memberUserData->pending) {
                $message = __("Your are not yet passed the guild's membership screening requirements", 'argus-wordpress-plugin');
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
        }

        // Getting user's guilds
        $userGuildsData = $pluginData->requestAllGuildsUserAreMemberOf(['argusDiscordOAuth2AccessToken' => $tokenResponse->access_token]);

        // Checking if required data is received
        if (!is_array($userGuildsData)) {
            $message = __("Discord server did not send a requested data", 'argus-wordpress-plugin');
            if (isset($userGuildsData->message)) {
                $message .= '. ' . __("Response message:", 'argus-wordpress-plugin');
                $message .= ' <i>' . $userGuildsData->message . '</i>';
            }
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Checking if user are a member of allowed guild
        if (get_option('argusDiscordGuildFilterMode') == 'allow') {
            $list    = explode(',', get_option('argusDiscordGuildFilter'));
            $allowed = false;
            foreach ($userGuildsData as $guild) {
                if (in_array($guild->id, $list)) {
                    $allowed = true;
                    break;
                }
            }
            if (!$allowed) {
                $message = __("You are not a member of allowed guild", 'argus-wordpress-plugin');
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
        }

        // Checking if user are a member of disallowed guild
        if (get_option('argusDiscordGuildFilterMode') == 'disallow') {
            $list    = explode(',', get_option('argusDiscordGuildFilter'));
            $allowed = true;
            $name    = '';
            foreach ($userGuildsData as $guild) {
                if (in_array($guild->id, $list)) {
                    $allowed = false;
                    $name    = $guild->name;
                    break;
                }
            }
            if (!$allowed) {
                $message = sprintf(__("Members of %s are not allowed", 'argus-wordpress-plugin'), '<i>' . $name . '</i>');
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
        }

        // Checking if roles filter is active
        if ((get_option('argusDiscordRolesFilterMode') == 'allow' || get_option('argusDiscordRolesFilterMode') == 'disallow') && !isset($memberUserData->roles)) {
            $message = __("Your are not a member of the guild", 'argus-wordpress-plugin');
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Checking if user has allowed role
        if (get_option('argusDiscordRolesFilterMode') == 'allow') {
            $list    = explode(',', get_option('argusDiscordRolesFilter'));
            $allowed = false;
            foreach ($memberUserData->roles as $role) {
                if (in_array($role, $list)) {
                    $allowed = true;
                    break;
                }
            }
            if (!$allowed) {
                $message = __("You do not have role required for access", 'argus-wordpress-plugin');
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
        }

        // Checking if user has disallowed role
        if (get_option('argusDiscordRolesFilterMode') == 'disallow') {
            $list    = explode(',', get_option('argusDiscordRolesFilter'));
            $allowed = true;
            $role_id = '';
            foreach ($memberUserData->roles as $role) {
                if (in_array($role, $list)) {
                    $allowed = false;
                    $role_id = $role;
                    break;
                }
            }
            if (!$allowed) {
                $name    = $pluginData->getRoleName($role_id);
                $message = sprintf(__("Members with role %s are not allowed", 'argus-wordpress-plugin'), '<i>' . $name . '</i>');
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
        }

    }


    static function modifyWPLoginForm () {
        if (get_option('argusDiscordModifyWPLoginForm')) {
            $link    = Data::getAuthorizationLink('argus-auth-discord-go-last-page');
            $styling = 'button button-primary button-large';
            include ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/login-form.phtml';
        }
    }


    static function modifyMemberPressLoginForm () {
        if (get_option('argusDiscordModifyMemberPressLoginForm')) {
            $link    = Data::getAuthorizationLink('argus-auth-discord-go-last-page');
            $styling = 'argus__authorization argus__button--default-style';
            include ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/login-form.phtml';
        }
    }


    static function addCommentWithDiscordButton () {
        if (!is_user_logged_in()) {
            $cpt = explode(',', get_option('argusDiscordCommentsCPT'));
            if (!count($cpt)) return;

            $style = '';
            if (get_option('argusDiscordStylingOAuthButtons')) {
                $style = 'default';
                wp_enqueue_style('argus-discord-front-end-styles');
            }

            if (in_array(get_post()->post_type, $cpt)) {
                echo do_shortcode('[argus-discord-authorization-button style="' . $style . '"]');
            }
        }
    }


}
