<?php

namespace Argus\Discord;

if (!defined('ABSPATH')) exit;


// Checking if component is installed
if (in_array('discord', ARGUS_ACTIVE_COMPONENTS)) {

    // Defining constants
    define('Argus\Discord\ARGUS_DISCORD_PLUGIN_PATH', ARGUS_PLUGIN_PATH . 'components/discord/');
    define('Argus\Discord\ARGUS_DISCORD_PLUGIN_URL', ARGUS_PLUGIN_URL . 'components/discord/');

    // Loading models
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'models/API.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'models/APIErrorHandler.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'models/Data.php';

    // Loading controllers
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Activate.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Deactivate.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Cron.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Authorization.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/RolesSync.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Announcement.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/AdminSettings.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Gutenberg.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/AccountPages.php';
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Avatars.php';

    // Loading ajax hooks
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/ajax-hooks.php';


    // Loading widgets
    include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/status/status.php';

    if (get_option('argusDiscordOAuth2AccessToken')) {

        // Registering front end styles
        add_action('wp_enqueue_scripts', function() {
            wp_register_style('argus-discord-front-end-styles', ARGUS_DISCORD_PLUGIN_URL . 'views/styles/frontend.css', [], ARGUS_PLUGIN_VERSION);
        });

        // Loading blocks
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/sharing/sharing-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/sharing/sharing-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/authorization/authorization-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/authorization/authorization-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/connect-profile/connect-profile-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/connect-profile/connect-profile-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/synchronization/synchronization-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/synchronization/synchronization-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/default-widget/default-widget-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/default-widget/default-widget-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/invite-banner/invite-banner-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/invite-banner/invite-banner-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/chat/chat-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/chat/chat-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/invite-link/invite-link-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/invite-link/invite-link-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/members-count/members-count-shortcode.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/members-count/members-count-gutenberg.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/profile-id/profile-id-shortcode.php';

        include_once ARGUS_DISCORD_PLUGIN_PATH . 'widgets/username/username-shortcode.php';


        // Registering announcements
        add_action('transition_post_status', ['Argus\Discord\Announcement', 'announceCPT'], 10, 3);
        add_action('woocommerce_new_order', ['Argus\Discord\Announcement', 'announceWoocommerceOrder'], 10, 1);

        // Registering account pages
        AccountPages::init();

        // Registering Discord avatars
        Avatars::init();

        // Registering gutenberg section
        Gutenberg::registerSection();

        // Registering elementor blocks
        if (ARGUS_ELEMENTOR_ACTIVE) {
            include_once ARGUS_DISCORD_PLUGIN_PATH . 'controllers/Elementor.php';
            Elementor::instance();
            Elementor::register_category();
            add_action('elementor/editor/before_enqueue_scripts', ['Argus\Discord\Elementor', 'connect_editor_assets']);
        }
    }


    // Registering plugin settings
    add_action('admin_init', function() {
        register_setting('argus-discord-oauth2-group', 'argusDiscordOAuth2ClientID');
        register_setting('argus-discord-oauth2-group', 'argusDiscordOAuth2ClientSecret');
        register_setting('argus-discord-oauth2-group', 'argusDiscordOAuth2BotToken');
    });


    // Initializing cron
    Cron::init();


    // Catching request to disconnect Discord profile
    if (isset($_GET['state']) && $_GET['state'] == 'argus-disconnect-discord-profile') {
        add_action('wp_loaded', ['Argus\Discord\Authorization', 'disconnectDiscordProfileFromUser']);
    }

    // Catching discord API authorization response
    if (isset($_GET['state']) && $_GET['state'] == 'argus-auth-discord-go-last-page') {
        add_action('wp_loaded', function() {
            Authorization::authorizeOrRegisterUser();
        }, 1, 0);
    }
    if (isset($_GET['state']) && $_GET['state'] == 'argus-auth-discord-go-prev-page') {
        add_action('wp_loaded', function() {
            Authorization::authorizeOrRegisterUser('argusPrevPage');
        }, 1, 0);
    }
    if (isset($_GET['state']) && preg_match('/^argus-auth-discord-go-([0-9]+)-page$/', $_GET['state'])) {
        add_action('wp_loaded', function() {
            Authorization::authorizeOrRegisterUser($_GET['state']);
        }, 1, 0);
    }

    if (isset($_GET['state']) && $_GET['state'] == 'argus-connect-discord-profile-to-user') {
        add_action('wp_loaded', ['Argus\Discord\Authorization', 'connectProfileToUser']);
    }

    // Minifying default WP login form
    add_action('login_form', ['Argus\Discord\Authorization', 'modifyWPLoginForm'], 100, 0);

    // Minifying MemberPress login form
    add_action('mepr-login-form-before-submit', ['Argus\Discord\Authorization', 'modifyMemberPressLoginForm'], 100, 0);

    // Modifying comment forms
    add_action('comment_form_top', ['Argus\Discord\Authorization', 'addCommentWithDiscordButton'], 100, 0);


    // Subscribing user roles synchronization
    add_action('wp_login', ['Argus\Discord\RolesSync', 'syncWPUserRoles'], 100, 2);

    if (!isset($_GET['action']) || ($_GET['action'] != 'argusDiscordSyncUserRoles' && $_GET['action'] != 'argusDiscordSyncAllUsersRoles')) {
        add_action('set_user_role', ['Argus\Discord\RolesSync', 'syncWPUserRolesAfterRoleChange'], 100, 1);
    }

    add_action('profile_update', ['Argus\Discord\RolesSync', 'syncWPUserRolesAfterRoleChange'], 100, 1);

    if (isset($_GET['action']) && $_GET['action'] == 'argus-synchronize-user') {
        add_action('wp_loaded', ['Argus\Discord\RolesSync', 'syncAuthorizedWPUserData']);
    }

    add_action('mepr-event-transaction-completed', function($event) {
        (new RolesSync())->syncWPUserRolesAfterRoleChange($event->get_data()->user()->ID);
    }, 100, 1);

    add_action('mepr-event-transaction-refunded', function($event) {
        (new RolesSync())->syncWPUserRolesAfterRoleChange($event->get_data()->user()->ID);
    }, 100, 1);

    add_action('mepr-transaction-expired', function($txn) {
        (new RolesSync())->syncWPUserRolesAfterRoleChange($txn->user_id);
    }, 100, 1);

    add_action('mepr-txn-store', function($txn) {
        if ($txn->is_expired()) {
            (new RolesSync())->syncWPUserRolesAfterRoleChange($txn->user_id);
        }
    }, 11, 1);

    add_action('mepr_post_delete_transaction', function($id, $user) {
        (new RolesSync())->syncWPUserRolesAfterRoleChange($user->ID);
    }, 100, 2);

    add_action('wcmo_after_update_users_role', function($user) {
        (new RolesSync())->syncWPUserRolesAfterRoleChange($user->ID);
    }, 100, 1);

    add_action('wcmo_after_expired_user_remove_role', function($user) {
        (new RolesSync())->syncWPUserRolesAfterRoleChange($user->ID);
    }, 100, 1);


    // Sharing
    if (isset($_GET['state']) && substr($_GET['state'], 0, 24) === 'argus-discord-share-page-') {
        add_action('wp_loaded', ['Argus\Discord\Authorization', 'shareContent']);
    }


    // Discord plugin authorization process. Checking Discord API Tokens
    if (isset($_GET['state']) && $_GET['state'] == get_option('argusDiscordOAuth2State')) {
        update_option('argusDiscordOAuth2State', '');
        Authorization::checkAuthorizationTokens();
    }

    // Admin panel actions
    if (is_admin()) {

        // Discord API authorization process actions. Component Authorization
        if (!get_option('argusDiscordOAuth2AccessToken') && get_option('argusDiscordOAuth2ClientID')) {
            Authorization::authorizePlugin();
        }

        // Discord API authorization process actions. Component logout
        if (isset($_GET['argus-logout'])) Data::authorizationLogout();


        // Subscribing user profile actions
        add_action('edit_user_profile', ['Argus\Discord\AdminSettings', 'modifyProfilePage']);
        add_action('show_user_profile', ['Argus\Discord\AdminSettings', 'modifyPersonalProfilePage']);
        add_action('edit_user_profile_update', ['Argus\Discord\AdminSettings', 'saveDataFromProfilePage']);
        add_action('personal_options_update', ['Argus\Discord\AdminSettings', 'saveDataFromProfilePage']);

        // Registering admin panel controls
        add_action('admin_menu', function() {
            $page = add_options_page(__("Discord Settings", 'argus-wordpress-plugin'), __("Discord", 'argus-wordpress-plugin'), 'manage_options', 'argus-discord-plugin-settings', ['Argus\Discord\AdminSettings', 'init']);

            add_action('load-' . $page, function() {
                wp_enqueue_media();
                wp_enqueue_style('argus-admin-panel-styles');
                wp_enqueue_script('argus-admin-helpers-scripts');

                if (get_option('argusDiscordOAuth2AccessToken')) {
                    $currentScreen = get_current_screen();
                    include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/help-tab.php';
                    include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/errors-help.php';
                    foreach (array_merge($tabs, $errorsHelp) as $tab) $currentScreen->add_help_tab($tab);
                    $currentScreen->set_help_sidebar($sidebar);
                }
            });
        });

        // Additional help tabs
        global $pagenow;
        if ($pagenow == 'user-edit.php') {
            add_action('admin_head', function() {
                $currentScreen = get_current_screen();
                include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/profile-help-tab.php';
                foreach ($tabs as $tab) $currentScreen->add_help_tab($tab);
            });
        }

        // Plugins manager page modifications
        add_filter('plugin_action_links_argus-wordpress-plugin', ['Argus\Discord\AdminSettings', 'modifyPluginsManagerPage']);

    }
}
