<?php

namespace Argus\Discord;

use Elementor\Controls_Manager;
use Elementor\Widget_Base;


class ElementorWidgetAuthorization extends Widget_Base {


    function get_name ():string {
        return 'argus_discord_authorization_widget';
    }


    function get_title ():string {
        return __("Discord Authorization Button", 'argus-wordpress-plugin');
    }


    function get_icon ():string {
        return 'argus-discord-icon-authorization-button';
    }


    function get_categories ():array {
        return ['argus-discord-category'];
    }


    protected function register_controls () {
        $redirectArray = json_decode(get_option('argusDiscordCustomRedirects'));

        $redirect_options = [
            'last' => __("Last page", 'argus-wordpress-plugin'),
            'prev' => __("Previous page", 'argus-wordpress-plugin'),
        ];
        foreach ($redirectArray as $id => $redirect) $redirect_options[ $id ] = $redirect->name;

        $this->start_controls_section('section_content', [
            'label' => __("Main", 'argus-wordpress-plugin'),
            'tab'   => Controls_Manager::TAB_CONTENT,
        ]);

        $this->add_control('login_text', [
            'label'   => __("Login text", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::TEXT,
            'default' => __("Login with Discord", 'argus-wordpress-plugin'),
        ]);

        $this->add_control('logout_text', [
            'label'   => __("Logout text", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::TEXT,
            'default' => __("Logout", 'argus-wordpress-plugin'),
        ]);

        $this->add_control('redirect_to', [
            'label'   => __("Redirect to", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::SELECT,
            'default' => 'last',
            'options' => $redirect_options,
        ]);

        $this->end_controls_section();

        $this->start_controls_section('section_style', [
            'label' => __("Settings", 'argus-wordpress-plugin'),
            'tab'   => Controls_Manager::TAB_STYLE,
        ]);

        $this->add_control('style', [
            'label'   => __("Style", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::SELECT,
            'default' => 'no',
            'options' => [
                'no'      => __("None", 'argus-wordpress-plugin'),
                'default' => __("Default", 'argus-wordpress-plugin'),
            ],
        ]);

        $this->end_controls_section();
    }


    protected function render () {
        $settings = $this->get_settings_for_display();

        if ($settings['style'] != 'no') wp_enqueue_style('argus-discord-front-end-styles');

        if (is_user_logged_in()) {
            $link = wp_logout_url($_SERVER['REQUEST_URI']);
        } else {
            $link = Data::getAuthorizationLink('argus-auth-discord-go-' . $settings['redirect_to'] . '-page');
        }

        if (isset($settings['style'])) {
            $class = 'argus-discord__button--' . $settings['style'] . '-style';
        } else {
            $class = 'argus-discord__button--no-style';
        }

        if (is_user_logged_in()) {
            $text = $settings['logout_text'] ?? __("Logout", 'argus-wordpress-plugin');
        } else {
            $text = $settings['login_text'] ?? __("Login with Discord", 'argus-wordpress-plugin');
        }

        ?>
        <a class="<?php echo $class; ?>" href="<?php echo $link; ?>"><?php echo $text; ?></a>
        <?php
    }


    protected function content_template () {
        ?>
        <a class="argus-discord__button--{{{ settings.style }}}-style" href="javascript:void(0)">
            {{{ settings.logout_text ? settings.logout_text : "<?php _e("Logout", 'argus-wordpress-plugin'); ?>" }}}
        </a>
        <?php
    }


}
