<?php

namespace Argus;

use DateTime;

abstract class AdminSettingsAbstraction {


    protected static function buildSupportDateTemplate ():string {
        $date     = Data::getSupportExpireDate();
        $template = '';

        if ($date) {
            if (new DateTime() > new DateTime($date)) {
                $template = "<a href='" . ARGUS_ENVATO_LINK . "' target='_blank'>" . __("Support expired", 'argus-wordpress-plugin') . " <code style='color:red'>" . $date . "</code>, " . __("click here to renew!", 'argus-wordpress-plugin') . "</a>";
            } else {
                $template = __("Supported up to", 'argus-wordpress-plugin') . " <code style='color:green'>" . $date . "</code>";
            }
        }

        return $template;
    }


    protected static function buildDynamicAdminColors ():array {
        global $_wp_admin_css_colors;
        $dynamicAdminColors = [];
        $color_scheme       = get_user_option('admin_color');
        if (!$color_scheme || !array_key_exists($color_scheme, $_wp_admin_css_colors)) {
            $color_scheme = array_key_first($_wp_admin_css_colors);
        }

        foreach ($_wp_admin_css_colors[ $color_scheme ]->colors as $key => $color) {
            $dynamicAdminColors[ $key ]['bg']   = $color;
            $dynamicAdminColors[ $key ]['text'] = self::getContrastColor($color);
        }
        return $dynamicAdminColors;
    }


    private static function getContrastColor ($hexColor):string {

        // Parsing hex color to RGB
        $R1 = hexdec(substr($hexColor, 1, 2));
        $G1 = hexdec(substr($hexColor, 3, 2));
        $B1 = hexdec(substr($hexColor, 5, 2));

        // Defining black color
        $blackColor   = '#000000';
        $R2BlackColor = hexdec(substr($blackColor, 1, 2));
        $G2BlackColor = hexdec(substr($blackColor, 3, 2));
        $B2BlackColor = hexdec(substr($blackColor, 5, 2));

        // Calculating contrast ratio
        $L1 = 0.2126 * pow($R1 / 255, 2.2) + 0.7152 * pow($G1 / 255, 2.2) + 0.0722 * pow($B1 / 255, 2.2);

        $L2 = 0.2126 * pow($R2BlackColor / 255, 2.2) + 0.7152 * pow($G2BlackColor / 255, 2.2) + 0.0722 * pow($B2BlackColor / 255, 2.2);

        if ($L1 > $L2) {
            $contrastRatio = (int)(($L1 + 0.05) / ($L2 + 0.05));
        } else {
            $contrastRatio = (int)(($L2 + 0.05) / ($L1 + 0.05));
        }

        // Defining contrast color
        if ($contrastRatio > 8) {
            return '#000000';
        } else {
            return '#FFFFFF';
        }
    }


    protected static function checkElementorVersion ():string {
        if (ARGUS_ELEMENTOR_ACTIVE && !version_compare(ELEMENTOR_VERSION, ARGUS_ELEMENTOR_VERSION, '>=')) {
            return sprintf(__("Plugin requires Elementor version %s or greater.", 'argus-wordpress-plugin'), ARGUS_ELEMENTOR_VERSION);
        }
        return '';
    }


    protected static function checkForCriticalErrors ($component = 0):string {
        $count = Log::countComponentCriticalErrors($component);
        if ($count) {
            return sprintf(__("Amount of errors that occurred during the interaction with API: %s. Click on this message to see the logs.", 'argus-wordpress-plugin'), $count);
        }
        return '';
    }

}
