<?php

namespace Argus;

use ZipArchive;


class Update extends AdminSettingsAbstraction {


    static function renderUpdateControls () {
        wp_enqueue_style('argus-admin-panel-styles');
        wp_enqueue_script('argus-update-panel', ARGUS_PLUGIN_URL . 'controllers/Update.js', ['jquery', 'react', 'react-dom'], ARGUS_PLUGIN_VERSION, true);

        if (!get_option('argusLastUpdateOnServer')) Data::getLastUpdateInfo();
        $updateData = get_option('argusLastUpdateOnServer');

        $dynamicColors = self::buildDynamicAdminColors();
        require ABSPATH . WPINC . '/version.php';

        $translationsFile = ARGUS_PLUGIN_PATH . '/languages/argus-react-' . get_locale() . '.json';

        ?>
        <script>
            var argusStateValues = {};

            <?php if ((int)get_option('argusActivateTranslations') && file_exists($translationsFile)) { ?>
            argusStateValues.translations = <?php echo file_get_contents($translationsFile); ?>;
            <?php } else { ?>
            argusStateValues.translations = false;
            <?php } ?>

            argusStateValues.colors = {
                accent: "<?php echo $dynamicColors[2]['bg']; ?>",
                text  : "<?php echo $dynamicColors[2]['text']; ?>"
            }

            argusStateValues.supportExpireDate       = "<?php echo self::buildSupportDateTemplate(); ?>";
            argusStateValues.supportEmail            = "<?php echo ARGUS_SUPPORT_EMAIL; ?>";
            argusStateValues.updateVersion           = "<?php echo $updateData->version ?? ''; ?>";
            argusStateValues.updateMessage           = "<?php echo $updateData->message ?? ''; ?>";
            argusStateValues.updateVersionPHP        = "<?php echo $updateData->php ?? ''; ?>";
            argusStateValues.updateVersionWP         = "<?php echo $updateData->wp ?? ''; ?>";
            argusStateValues.updateVersionElementor  = "<?php echo $updateData->elementor ?? ''; ?>";
            argusStateValues.currentVersion          = "<?php echo ARGUS_PLUGIN_VERSION; ?>";
            argusStateValues.currentVersionPHP       = "<?php echo phpversion(); ?>";
            argusStateValues.currentVersionWP        = "<?php echo $wp_version; ?>";
            argusStateValues.currentVersionElementor = "<?php echo ARGUS_ELEMENTOR_ACTIVE ? ELEMENTOR_VERSION : ''; ?>";
            argusStateValues.newVersionAvailable     = <?php echo self::checkForUpdate() ? 'true' : 'false'; ?>;
            argusStateValues.PHPVersionSupported     = <?php echo version_compare(phpversion(), $updateData->php, '>=') ? 'true' : 'false'; ?>;
            argusStateValues.WPVersionSupported      = <?php echo version_compare($wp_version, $updateData->wp, '>=') ? 'true' : 'false'; ?>;
            <?php if (ARGUS_ELEMENTOR_ACTIVE) { ?>
            argusStateValues.ElementorVersionSupported = <?php echo version_compare(ELEMENTOR_VERSION, $updateData->elementor, '>=') ? 'true' : 'false'; ?>;
            <?php } else { ?>
            argusStateValues.ElementorVersionSupported = true;
            <?php } ?>
            argusStateValues.WPContentWritable = <?php echo is_writable(WP_CONTENT_DIR) ? 'true' : 'false'; ?>;
        </script>

        <h2><?php _e("Argus Components", 'argus-wordpress-plugin'); ?></h2>
        <div id="argus-update-panel"></div>
        <?php
    }


    static function checkForUpdateCounter () {
        if (self::checkForUpdate()) {
            add_filter('wp_get_update_data', function($update_data) {
                $update_data['counts']['total'] ++;
                return $update_data;
            }, 1, 1);
        }
    }


    static function checkForUpdate () {
        if (!get_option('argusLastUpdateOnServer')) Data::getLastUpdateInfo();
        if (!get_option('argusLastUpdateOnServer')) return false;
        return version_compare(ARGUS_PLUGIN_VERSION, get_option('argusLastUpdateOnServer')->version, '<');
    }


    static function downloadUpdateZip ($zipUrl) {
        return file_put_contents(wp_upload_dir()['basedir'] . '/tmp.zip', fopen($zipUrl, 'r'));
    }


    static function unzipUpdateZip ():bool {
        $pluginRoot = WP_PLUGIN_DIR . '/argus-wordpress-plugin';

        // Renaming current folder
        if (rename($pluginRoot, $pluginRoot . '-old')) {

            // Unzipping archive
            $zip    = new ZipArchive;
            $result = $zip->open(wp_upload_dir()['basedir'] . '/tmp.zip');
            if ($result === true) {
                $result = $zip->extractTo(WP_PLUGIN_DIR);
                $zip->close();

                // Rolling back if unzip failed
                if (!$result) {
                    self::deleteFolder($pluginRoot);
                    rename($pluginRoot . '-old', $pluginRoot);
                    return false;
                }

                return true;
            }

        }
        return false;
    }


    static function clearUpdateGarbage ():bool {
        unlink(wp_upload_dir()['basedir'] . '/tmp.zip');
        self::deleteFolder(WP_PLUGIN_DIR . '/argus-wordpress-plugin-old');
        return true;
    }


    static function update ():bool {
        update_option('argusLastUpdateOnVersion', ARGUS_PLUGIN_VERSION);

        global $wpdb;

        return true;
    }


    private static function deleteFolder ($dir) {
        $files = array_diff(scandir($dir), ['.', '..']);
        foreach ($files as $file) {
            (is_dir("$dir/$file")) ? self::deleteFolder("$dir/$file") : unlink("$dir/$file");
        }
        return rmdir($dir);
    }


    /*
     * TODO: Delete when old plugin transition no longer be relative
     */
    static function subscribeOldPluginNotice () {
        if (in_array('discord-wordpress-plugin', scandir(WP_PLUGIN_DIR))) {
            add_action('load-settings_page_dwpp-discord-plugin-settings', ['Argus\Update', 'oldPluginNotice'], 999999999);
            add_action('pre_current_active_plugins', ['Argus\Update', 'oldPluginNotice'], 999999999);
            add_action('welcome_panel', ['Argus\Update', 'oldPluginNotice'], 999999999);
        }
    }


    /*
     * TODO: Delete this method with self::subscribeOldPluginNotice()
     */
    static function oldPluginNotice () {
        ?>
        <div class="notice notice-error dwpp-notice">
            <p style="font-size: 1.5em;">
                <b>ATTENTION!</b>
                The old version of the Argus plugin is installed. Old versions are not backward compatible and should not be used in parallel at the same time! To know more about the transition to the new version read
                <i style="color: red">Update from 5.x to 6.x</i>
                guide in the documentation of the new plugin.
            </p>
        </div>
        <?php
    }

}
